/*

Die Funktionalitaet von JSONFeed.js und TelekomCrypto.js zusammengefasst. Also einen
oder mehrere JSON Feeds parallel vom Telekom Proxy Server ziehen. Zeigt waehrend der 
Ausfuehrung den TelekomActivityIndicator an.

Methoden:

	getOneAsync(decURL, onRes)
		erhaelt einen Klartext-URL, encryptet diesen und erzeugt das Auth Token, ruft
		den Telekom Proxy damit auf und gibt das um Kontext Informationen angreicherte 
		JSON Objekt an demn Callback zurueck.
		Parameter:
			decURL
				der Klartext URL
			onRes	
				der Callback	 			
			Rueckgabewerte (gekapselt im Rueckgabeobjekt):
				- dec
					der Klartext-URL
				- enc
					der encryptete URL
				- jqXHR
					das JQuery Kontext Objekt
				- textStatus
					der ResponseCode			
				- data
					das JSON Daten Objekt - fehlt bei Fehler
				- errorThrown
					das Fehler Objekt - fehlt bei Erfolg
			
	getManyAsync(decURLs, onRess)
		erhaelt einen Array von Klartext-URLs und ruft getOneAsync mit jedem davon auf.
		Erzeugt einen Sammelcallback, der als 2. Parameter an all diese Aufrufe uebergeben 
		wird. Dieser nimmt alle Rueckgabeaufrufe der EinzelAufrufe entgegen, sammelt diese
		und ruft, wenn alle geantwortet haben, den Callback mit einem Array der 
		EinzelRueckgabewerte auf. Verhindert so RaceConditions, die entstehen wuerden wenn
		mehrere Callbacks Asynchron an einem einzigen Datenobjekt herumschreiben wueden.
		Die Reihenfolge der Rueckgabewerte im RueckgabeArray entspricht der Reihenfolge
		der Klartext-URL im Klartext-URL-Array decURLs.
		Parameter:
			decURLs
				vgl. getOneAsync und decURL - nur ein Array davon
			onRess
				vgl. getOneAsync und onRes - nur ein Array von Rueckgabewerten

*/

var TelekomJSONFeed = function(){

	// Const
	var TIMEOUT_MILLIS = 60000; 
	
	var getOneAsync = function(decURL, onRes){
		TelekomActivityIndicator.inc();
		var encURL  = TelekomCrypto.getURLString(decURL);
		var urls    = {
			dec:decURL,
			enc:encURL
		};	
		var onTelekomRes = function(telekomRes){
			telekomRes.urls = urls     ;
			telekomRes.url  = undefined;
			onRes(telekomRes);			
			TelekomActivityIndicator.dec();
		};
		JSONFeed.getAsync(encURL, onTelekomRes, TIMEOUT_MILLIS);
	}
	var getManyAsync = function(decURLs, onRess){
		var ressUnsorted = [];
		var ressSorted   = [];
		var onOneRes     = function(res){
			ressUnsorted.push(res);
			if(ressUnsorted.length == decURLs.length){
				$.each(decURLs, function(indexOfDecURL, decURL){
					$.each(ressUnsorted, function(indexOfResUnsorted, resUnsorted){
						var resUnsortedDecURL = resUnsorted.urls.dec;
						if(resUnsortedDecURL == decURL){
							ressSorted.push(resUnsorted);
						}
					});
				});
				onRess(ressSorted);
			}
		}
		$.each(decURLs, function(indexOfDecURL, decURL){
			getOneAsync(decURL, onOneRes);
		});
	}
	var TelekomJSONFeed_ = {
		getOneAsync : getOneAsync ,
		getManyAsync: getManyAsync
	}
	return TelekomJSONFeed_;
}();
