/*

Die Klasse zum Verwalten der Settings. Inbesondere zum persistenten Speichern dieser 
Settings und zum Registrieren der Feeds bei UrbanAirship.

Methoden:

	registerAsync(callBack)
		registriert bei UrbanAirship die gewuensachten Feeds. Uebergibt ein Objekt mit
		diesen Feeds an den callBack
	
	setShouldBeDisplayed(outlineID, shouldBeDisplayed)
		stellt ein, ob ein Feed angezeigt werden soll

	setShouldReceivePush(outlineID, shouldBeDisplayed)
		stellt ein, ob PushNotifications zu einem Feed empfangen werden sollen und 
		uebermittelt das an Urban Airship.
		
	update(outlines)
		aktualisiert die Settings bei neuen OPML-Outlines. Sprich: setze die Deafult-Werte
		fuer eventuelle neue Feeds. 

*/

var TelekomSettings = function(){
	//Const 
	var telekomSettingsFileName = 'TelekomSettings.json';
	var TelekomSettings_        = {};

	var registerAsync = function(){
		var oldTags = [''];
		var registerAsync_ = function(callBack){
			var registerAsyncCallBack = function(jsonObj){
				if(!jsonObj){jsonObj = {}};
				var tags = [];
				$.each(jsonObj, function(tag, tagData){
					if(tagData.shouldReceivePush){
						tags.push(tag);
					}	
				});
				
				/*
					Hier wird die Differenz zwischen den alten und den neuen Tags 
					gebildet. Verfahren:
						1. Erzeuge eine Map, die alle alten und alle neuen Tags als Keys
						und als Werte immer die 0 enthaet.
						2. Addiere fuer alle neuen Tags 1 zu dem Wert unter diesem Tag
						3. Sutrahiere fuer alle alten Tags 1 zu dem Wert unter diesem Tag
						4. guck aller Werte durch, on eine nicht 0 ist. Wenn ja, hat sich
							was geaendert und man muss sich neu registrieren
				*/
													
				var didTagsChange = false;
				var tagsCmp       = {}   ;
				$.each(tags   , function(indexOfTag, tag){tagsCmp[tag]  = 0;});
				$.each(oldTags, function(indexOfTag, tag){tagsCmp[tag]  = 0;});
				$.each(tags   , function(indexOfTag, tag){tagsCmp[tag] += 1;});
				$.each(oldTags, function(indexOfTag, tag){tagsCmp[tag] -= 1;});
				$.each(tagsCmp, function(tag, tagCmp){

				if(tagCmp){
					TKConnector.setTags(tags)
				}

					oldTags = tags;
				});	
				callBack(jsonObj);

			}
			registerAsyncCallBack(getSettings());
		}
		return registerAsync_;
	}();
	
	var getJSONObject = function(){
		var jsonString = localStorage.getItem(telekomSettingsFileName);
		var jsonObject = MKJSON.toObject(jsonString);
		if(!jsonObject){
			jsonObject = {};
		}	
		return jsonObject;
	}
		 
	var setJSONObject = function(jsonObject){
		var jsonString = MKJSON.toString(jsonObject);
		localStorage.setItem(telekomSettingsFileName, jsonString);
	}	

	var getSettings = function(){
		var settings = getJSONObject();
		if(!settings){
			settings = {};
		}
		return settings;
	}

	var setSetting = function(outlineID, settingKey, settingValue){
		var settings        = getSettings();
		var outlineSettings = settings[outlineID];
		if(!outlineSettings){
			outlineSettings     = {};
			settings[outlineID] = outlineSettings;
		}
		outlineSettings[settingKey] = settingValue;
		setJSONObject(settings);
		registerAsync(function(){});
	}
	
	var setShouldBeDisplayed = function(outlineID, shouldBeDisplayed){setSetting(outlineID, 'shouldBeDisplayed', shouldBeDisplayed);}
	var setShouldReceivePush = function(outlineID, shouldReceivePush){setSetting(outlineID, 'shouldReceivePush', shouldReceivePush);}

	var update = function(outlines){
		var settings = getSettings();
		$.each(outlines, function(indexOfOutline, outline){
			var outlineID = outline.id;
			if(!settings[outlineID]){
				setShouldBeDisplayed(outlineID, true);
				setShouldReceivePush(outlineID, true);
			}
		});
		settings = getSettings();
		return settings;
	}	
	
	var clear = function(){
		setJSONObject({})
	}	

	TelekomSettings_.getSettings          = getSettings         ;
	TelekomSettings_.registerAsync        = registerAsync       ;
	TelekomSettings_.setShouldBeDisplayed = setShouldBeDisplayed;
	TelekomSettings_.setShouldReceivePush = setShouldReceivePush;
	TelekomSettings_.update               = update              ;
	TelekomSettings_.clear                = clear               ;

	return TelekomSettings_;
}();
